<?php
class Ameex_Insurance_ClaimController extends Mage_Core_Controller_Front_Action
{
	public function indexAction()
	{
		$data = $this->getRequest()->getParams();
		$imei = $data[device];
		$activationData = Mage::getModel("ameex_insurance/activationDetails")->getCollection()->addFieldToFilter("imei",$imei);
		if(count($activationData) == 0) {

			/********************
			*	IMEI not activated
			********************/
			
			Mage::getSingleton("core/session")->addError("The IMEI number that you have entered is not yet activated for insurance.");
        	$this->_redirect('*/*/result');
		}
		else {

			/********************
			*	IMEI activated
			********************/

			$activation = $activationData->getFirstItem();
			$purchasedOn = $activation->getCreatedAt();
			$activatedOn = $activation->getActivatedAt();
			$timeTakenForActivation = strtotime($activatedOn) - strtotime($purchasedOn);
			$actualActivationDate = ($timeTakenForActivation <= 2592000) ? $activatedOn : (strtotime($purchasedOn) + 2592000);
			$insurancePeriodExceeded = ((time() - strtotime($actualActivationDate)) > 31536000) ? true : false;

			if ($insurancePeriodExceeded) {
				
				/********************
				*	Outside Insurance period
				********************/
				Mage::getSingleton("core/session")->addError("Your insurance period expired.");
        		$this->_redirect('*/*/result');
				
			} else {
				
				/********************
				*	Within the Insurance period
				********************/

				$isMinimumPeriodExceeded = ((time() - strtotime($actualActivationDate)) > 2592000) ? true : false;
				if ($isMinimumPeriodExceeded) {
					
					/********************
					*	Minimum waiting period exceeded
					********************/
					
					$transactionStatus = $this->_processCard($data);
					if ($transactionStatus[0] == 1) {
						
						/********************
						*	Transaction success.
						********************/

						$claim = Mage::getModel("ameex_insurance/claimDetails");
						$claim->setFirstName($data["first-name"]);
					    $claim->setLastName($data["last-name"]);
					    $claim->setEmail($data["email"]);
					    $claim->setDeviceNumber($data["device"]);
					    $claim->setPrimaryPhone($data["primary-phone"]);
					    $claim->setAlternatePhone($data["alternate-phone"]);
					    $claim->setAddress1($data["address-1"]);
					    $claim->setCity($data["city"]);
					    $claim->setState($data["state"]);
					    // $claim->setPolicyNumber($data["policy"]);
					    $claim->setAddress2($data["address-2"]);
					    $claim->setZipcode($data["zip"]);
					    $claim->setCountry($data["country"]);
					    $claim->setDamagedAt($data["date-of-damage"]);
					    $claim->setDamageType($data["type-of-damage"]);
					    $claim->setDeviceWork($data["does-your-device-still-work"]);
					    $claim->setDescription($data["description-of-incident"]);
					    ($data["additional-5-95-charge"] == "OK") ? $claim->setAmountPaid(30.95) : $claim->setAmountPaid(25);
					    $claim->setAgreeAdditionalCharge($data["additional-5-95-charge"]);
					    $claim->setClaimedAt(time());
					    $claim->save();

					    /********************
						*	To customer
						********************/
						
						$customerEmail = $claim->getEmail();
						// mail($customerEmail, 'We received your claim request.', "We received your claim request.");

						/********************
						*	To Insurance Company
						********************/
						
						$activationDetailsForThisImei = Mage::getModel("ameex_insurance/activationDetails")->load($claim->getDeviceNumber(),"imei");
						$insuranceCompany = Mage::getModel("ameex_insurance/company")->load($activationDetailsForThisImei->getInsuranceCompanyId());
						// mail($insuranceCompany->getEmail(), 'One claim request arrived', "One claim request arrived");

						/********************
						*	To admin
						********************/
						
						$adminEmail = Mage::getStoreConfig('trans_email/ident_general/email');
						$additionalCharge = ($data["additional-5-95-charge"] == "OK") ? "Customer agreed for additional charge." : "Customer not agreed for additional charge.";
						// mail($adminEmail, "One claim request arrived and the Insurance company notified. ".$additionalCharge, "One claim request arrived and the Insurance company notified. ".$additionalCharge);

					    Mage::getSingleton("core/session")->addSuccess("Thanks for your payment. Our support team will contact you at the earliest.");
			        	$this->_redirect('*/*/result');
					} else {
						
						/********************
						*	Transaction failed.
						********************/

						$error = "There was an error processing your request. Please try again or contact our support. We thank you!<br />Error: ".$transactionStatus[3];
						Mage::getSingleton("core/session")->addError($error);
			        	$this->_redirect('*/*/result');	
					}	
				} else {
					
					/********************
					*	Still within the Minimum waiting period
					********************/
					
					Mage::getSingleton("core/session")->addError("You have activated your ShieldX2 protection on ".date('m/d/Y',strtotime($actualActivationDate)).". Before submitting a claim, you have to wait for 30 days after Activation.");
			        $this->_redirect('*/*/result');	
				}
			}			
		}
	}

	public function resultAction()
	{
		$this->loadLayout();
		$this->renderLayout();		
	}

	private function _processCard($data) {
		$seed = time();
        $post_values = array(
            "x_login" => "87tmGXBU8QH",
            "x_tran_key" => "96sj8M5s8PD87d2C",
            "x_version" => "3.1",
            "x_delim_data" => "TRUE",
            "x_delim_char" => "|",
            "x_relay_response" => "FALSE",
            "x_type" => "AUTH_CAPTURE",
            "x_method" => "CC",
            "x_card_num" => $data["number-cc"],
            "x_exp_date" => $data["exp-month-cc"] . $data["exp-year-cc"],
            "x_amount" => ($data["additional-5-95-charge"] == "OK") ? "30.95" : "25.00",
            "x_description" => "Claim #: " . $seed,
            "x_first_name" => $data["first-name-cc"],
            "x_last_name" => $data["last-name-cc"],
            "x_address" => $data["address-1"] . " " . $data["address-2"],
            "x_state" => $data["state"],
            "x_zip" => $data["zip"],
            "x_country" => $data["country"]
        );
        $post_string = "";
        foreach($post_values as $key=>$value) {
            $post_string .= "$key=" . urlencode($value) . "&";
        }
        $post_string = rtrim($post_string,"& ");
        //$request = curl_init("https://secure.authorize.net/gateway/transact.dll");
        $request = curl_init("https://test.authorize.net/gateway/transact.dll");
        curl_setopt($request,CURLOPT_HEADER,0);
        curl_setopt($request,CURLOPT_RETURNTRANSFER,1);
        curl_setopt($request,CURLOPT_POSTFIELDS,$post_string);
        curl_setopt($request,CURLOPT_SSL_VERIFYPEER,FALSE);
        $post_response = curl_exec($request);
        curl_close($request);
        $response_array = explode($post_values["x_delim_char"],$post_response);
        return $response_array;
    }
}